/**
 * Created by PC on 2017/11/23.
 */
import Drag from 'draggable-move';

const ElZoom = function (options) {
    if (!options.selector) throw new Error("drag selector id is required ! ");
    this.el = typeof options.selector === 'string' ? document.getElementById(options.selector) : options.selector;
    if (typeof this.el !== 'object' || this.el.nodeType !== 1) throw Error("drag selector el is not a node element !");
    this.wrapper = document.documentElement || document.body;
    if (options.wrapper) {
        this.wrapper = typeof options.wrapper === 'string' ? getElementById(options.wrapper) : options.wrapper;
    }
    if (typeof this.wrapper !== 'object' || this.wrapper.nodeType !== 1) throw Error("move wrapper el is not a node element !");
    this.min = {
        width: this.el.clientWidth,
        height: this.el.clientHeight
    };
    if (options.min && options.min.width && options.min.height) {
        this.min = options.min
    }
    this.init(options);
};

ElZoom.prototype = {
    init: function (options) {
        var _this = this;
        this.el.className += ' resize-box';
        this.resizeCallBack = typeof options.resize === 'function' ? options.resize : ()=> {
            console.log("drag resize ... ", this);
        };
        this.move = typeof options.move === 'function' ? options.move : ()=> {
            console.log("drag moving ... ", this);
        };

        this.dragObj = new Drag({
            el: this.el,
            wrapper: this.wrapper,
            move: function (pos, drag) {
                const el = drag.scope;
                let _left = el.offsetLeft + pos.moveX;
                let _top = el.offsetTop + pos.moveY;
                _top = Math.min(drag.moveArea.y, Math.max(0, _top));
                _left = Math.min(drag.moveArea.x, Math.max(0, _left));
                el.style.left = _left + 'px';
                el.style.top = _top + 'px';
                _this.move.apply(null, [_this.dragObj]);
            }
        });

        this.createReact();
    },
    createReact: function () {
        var _this = this;
        var rect = ['resize-T', 'resize-R', 'resize-B', 'resize-L', 'resize-TL', 'resize-TR', 'resize-BR', 'resize-BL'];
        var rectLen = rect.length;
        for (var i = 0; i < rectLen; i++) {
            var tempEl = document.createElement('div');
            tempEl.className = rect[i];
            this.el.appendChild(tempEl);
            new Drag({
                selector: tempEl,
                wrapper: this.wrapper,
                move: function (pos, drag) {
                    _this.resize(drag.el.className, pos, drag);
                }
            });
        }
    },
    resize: function (dir, pos, drag) {
        var _this = this;
        var getPrototypeVal = function (val) {
            return (val + '').replace("px", '') * 1;
        };
        var setScopeSize = function (pro, val, changePro) {
            var proMap = {
                'width': 'clientWidth',
                'height': 'clientHeight',
                'top': 'offsetTop',
                'left': 'offsetLeft'
            };
            if (changePro) {
                var oldChangePro = getPrototypeVal(_this.el[proMap[changePro]]);
                _this.el.style[changePro] = oldChangePro - val + 'px';
            }
            _this.el.style[pro] = Math.max(_this.min[pro], _this.el[proMap[pro]] + val) + 'px';
            _this.resizeCallBack.apply(null, [_this]);
        };
        var resizeBoxSize = {
            width: this.el.clientWidth,
            height: this.el.clientHeight
        };
        var resizeBoxPos = {
            top: this.el.offsetTop,
            left: this.el.offsetLeft
        };
        var wrapperBoxSize = {
            width: this.wrapper.offsetWidth,
            height: this.wrapper.offsetHeight
        };
        var resizeBoxMaxReduceSize = {
            width: this.min.width - resizeBoxSize.width,
            height: this.min.height - resizeBoxSize.height
        };
        var getSettingResize = function (val, max, min) {
            return Math.max(Math.min(val, max), min);
        };
        switch (dir) {
            case "resize-T":
                setScopeSize('height', getSettingResize(-pos.moveY, resizeBoxPos.top, resizeBoxMaxReduceSize.height), 'top');
                break;
            case "resize-L":
                setScopeSize('width', getSettingResize(-pos.moveX, resizeBoxPos.left, resizeBoxMaxReduceSize.width), 'left');
                break;
            case "resize-R":
                setScopeSize('width', getSettingResize(pos.moveX, wrapperBoxSize.width - resizeBoxPos.left - resizeBoxSize.width, resizeBoxMaxReduceSize.width));
                break;
            case "resize-B":
                setScopeSize('height', getSettingResize(pos.moveY, wrapperBoxSize.height - resizeBoxPos.top - resizeBoxSize.height, resizeBoxMaxReduceSize.height));
                break;
            case "resize-TL":
                setScopeSize('width', getSettingResize(-pos.moveX, resizeBoxPos.left, resizeBoxMaxReduceSize.width), 'left');
                setScopeSize('height', getSettingResize(-pos.moveY, resizeBoxPos.top, resizeBoxMaxReduceSize.height), 'top');
                break;
            case "resize-TR":
                setScopeSize('width', getSettingResize(pos.moveX, wrapperBoxSize.width - resizeBoxPos.left - resizeBoxSize.width, resizeBoxMaxReduceSize.width));
                setScopeSize('height', getSettingResize(-pos.moveY, resizeBoxPos.top, resizeBoxMaxReduceSize.height), 'top');
                break;
            case "resize-BR":
                setScopeSize('width', getSettingResize(pos.moveX, wrapperBoxSize.width - resizeBoxPos.left - resizeBoxSize.width, resizeBoxMaxReduceSize.width));
                setScopeSize('height', getSettingResize(pos.moveY, wrapperBoxSize.height - resizeBoxPos.top - resizeBoxSize.height, resizeBoxMaxReduceSize.height));
                break;
            case "resize-BL":
                setScopeSize('width', getSettingResize(-pos.moveX, resizeBoxPos.left, resizeBoxMaxReduceSize.width), 'left');
                setScopeSize('height', getSettingResize(pos.moveY, wrapperBoxSize.height - resizeBoxSize.height - resizeBoxPos.top, resizeBoxMaxReduceSize.height));
                break;
        }
    }
};

export  default ElZoom;
